#include <stdio.h>
#include <stdlib.h>
#include <stralloc.h>
#include <string.h>
#include "person.h"
#include "hash.h"
#include "readline.h"

void show_name(struct Person* person) {
   printf("   %s", person->name.s);
   if (person->named) {
      printf(" (already named)");
   }
   printf("\n");
}

struct Person* search_person(PersonHashTable* pht, const char* name,
      unsigned short begin_year, unsigned short end_year) {
   const char* lastname = strrchr(name, ' ');
   if (lastname) {
      ++lastname;
   } else {
      lastname = name;
   }
   pht_first(pht, lastname);
   unsigned int given_name_len = lastname - name;
   struct Person* found = 0;
   struct Person* person = 0;
   bool ambigious = false;
   while ((person = pht_next(pht))) {
      if (name != lastname &&
	    strncmp(name, person->name.s, given_name_len) != 0) {
	 continue;
      }
      if (strcmp(name, person->name.s) == 0) {
	 // exact match
	 found = person;
	 ambigious = false;
	 break;
      }
      if (found) {
	 printf("%s is ambiguous. I got following names:\n", name);
	 show_name(found);
	 ambigious = true;
	 found = 0;
      }
      if (ambigious) {
	 show_name(person);
	 continue;
      }
      found = person;
   }
   if (found && begin_year && end_year &&
	 (found->died < begin_year || found->born > end_year)) {
      printf("%s did not live in the time span of %d-%d\n",
	 name, begin_year, end_year);
      found = 0;
   } else if (!found && !ambigious) {
      printf("Nobody found with that name.\n");
   }
   return found;
}

int main() {
   /* load authors from authors.txt */
   PersonHashTable* pht = pht_create();
   if (!pht) exit(1);
   FILE* fp = fopen("authors.txt", "r");
   if (!fp) exit(2);

   unsigned int count = 0;
   unsigned int min_year = 0;
   unsigned int max_year = 0;
   struct Person* p;
   while ((p = read_person(fp))) {
      if (!min_year || p->born < min_year) min_year = p->born;
      if (!max_year || p->died > max_year) max_year = p->died;
      pht_add(pht, p); ++count;
   }
   printf("%d dead authors loaded", count);
   if (min_year && max_year) {
      printf(" who lived in the time from %d to %d", min_year, max_year);
   }
   printf(".\n");

   printf(
      "*** Dead Author Chain Game ***\n"
      "Start by naming an author and then name another one\n"
      "whose live span intersects that of the former author.\n"
      "You get a point for each extension of the chain. No author\n"
      "may be named twice.\n"
      "Just type in the family name of an author if you do not\n"
      "remember the given name.\n"
   );

   unsigned int chain_count = 0;
   stralloc line = {0};
   struct Person* last_person = 0;
   while (printf("Name: "), readline(stdin, &line)) {
      stralloc_0(&line);
      struct Person* p;
      if (last_person) {
	 p = search_person(pht, line.s, last_person->born, last_person->died);
      } else {
         p = search_person(pht, line.s, 0, 0);
      }
      if (p) {
	 if (p->named) {
	    printf("Already named: ");
	 } else {
	    p->named = true;
	    if (last_person) ++chain_count;
	    last_person = p;
	 }
	 write_person(stdout, p);
      }
   }
   if (chain_count) {
      printf("You got %d points.\n", chain_count);
   }

   pht_free(pht);
}
